import json
import logging
from enum import StrEnum
from functools import cache

import requests

logger = logging.getLogger(__name__)


class EventType(StrEnum):
    NEW = "new"
    UPDATE = "update"
    END = "end"


class FrigateConfig:
    def __init__(self, base_url: str = "http://frigate:5000") -> None:
        self._base_url = base_url
        self._camera_zones = {}

    def fetch(self) -> None:
        response = requests.get(self._base_url)
        response.raise_for_status()
        config = json.loads(response.content)
        self._camera_zones = {
            camera_name: {
                required_zone: {
                    object_label
                    for object_label in camera_config["zones"][required_zone]["objects"]
                }
                for required_zone in camera_config["review"]["alerts"]["required_zones"]
            }
            for camera_name, camera_config in config["cameras"].items()
        }
        logger.info("Fetched Frigate camera zone configuration")

    @cache
    def is_alert(self, event_type: EventType, camera: str, new_zone: str, object_label: str) -> bool:
        if zones := self._camera_zones[camera]:
            if (object_labels := zones.get(new_zone)) is None:
                # New zone is not a required zone
                return False
            else:
                if object_labels:
                    # Alert if event has required object label
                    return object_label in object_labels
                else:
                    # No required object labels
                    return True
        else:
            # No required zones, new event is an alert
            return event_type == EventType.NEW
