import datetime as dt
import time
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import StrEnum
from functools import cache, cached_property


class MobileOS(StrEnum):
    IOS = "ios"
    ANDROID = "android"


@dataclass
class Notification:
    event_id: str
    camera: str
    object_label: str
    score: float

    @cached_property
    def camera_display_name(self) -> str:
        return " ".join(word.capitalize() for word in self.camera.split("_"))

    @cached_property
    def camera_url(self) -> str:
        return f"https://frigate.homelab.net/cameras/{self.camera}"

    @cache
    def get_image_url(self, mobile_os: MobileOS = MobileOS.ANDROID) -> str:
        return f"https://frigate.homelab.net/api/events/{self.event_id}/thumbnail.jpg?format={mobile_os}"


class NotificationSender[N: Notification](ABC):
    def __init__(self, quiet_period: dt.timedelta) -> None:
        self._quiet_period = quiet_period
        self._camera_to_last_notification_time = {}

    @abstractmethod
    def _send(self, notification: N) -> None:
        ...

    def send(self, notification: N) -> None:
        now = time.time()
        last_notification_time = self._camera_to_last_notification_time.setdefault(notification.camera, 0)
        if now - last_notification_time >= self._quiet_period.total_seconds():
            # Quiet period has passed since the last notification for this camera
            self._send(notification)
